/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/gpl-3.0.txt
 *
 */
package com.frojasg1.chesspdfbrowser.pdf.textsegmentation;

import com.frojasg1.chesspdfbrowser.pdf.textsegmentation.implementation.ImageSegmentator;
import com.frojasg1.general.number.IntegerFunctions;

/**
 *
 * @author Usuario
 */
public class PixelComponents
{
	public short _red;
	public short _green;
	public short _blue;
		
	public PixelComponents( short red, short green, short blue )
	{
		_red = red;
		_green = green;
		_blue = blue;
	}
	
	public short getRed()
	{
		return( _red );
	}
	
	public short getGreen()
	{
		return( _green );
	}
	
	public short getBlue()
	{
		return( _blue );
	}
	
	public boolean nearlyEquals( short[] pixelComponents, int tolerance )
	{
		boolean result = false;

		if( pixelComponents != null )
		{
			int redDiff = IntegerFunctions.abs( pixelComponents[0] - getRed() );
			int greenDiff = IntegerFunctions.abs( pixelComponents[1] - getGreen() );
			int blueDiff = IntegerFunctions.abs( pixelComponents[2] - getBlue() );

			if( ( getRed() >= 0 ) && ( redDiff <= tolerance ) &&
				( getGreen() >= 0 ) && ( greenDiff <= tolerance ) &&
				( getBlue() >= 0 ) && ( blueDiff <= tolerance ) )
			{
				result = true;
			}
		}

		return( result );
	}
	
	public int getPixelValue()
	{
		int pixelValue = ( ( ( (int) getRed() ) << 16 ) |
							( ( (int) getGreen() ) << 8 ) |
							( (int) getBlue() ) );
		return( pixelValue );
	}
}
