/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/gpl-3.0.txt
 *
 */
package com.frojasg1.chesspdfbrowser.application;

import com.frojasg1.applications.common.configuration.application.BaseApplicationConfigurationInterface;
import com.frojasg1.general.desktop.view.about.GenericAboutJDialog;
import com.frojasg1.general.desktop.view.pdf.ImageJPanel;
import java.awt.Dimension;
import java.awt.Point;
import java.io.File;

/**
 *
 * @author Usuario
 */
public class AboutJDialog extends GenericAboutJDialog
{
	protected static final String CONFIGURATION_BASE_FILE_NAME = "JDial_about";
	protected static final String RESOURCE_FOR_ABOUT_ICON = "com/frojasg1/chesspdfbrowser/resources/icons/App.icon.faded.png";
	public static final String ABOUT_RTF_FILE_NAME = "AboutText.rtf";

	protected static final String CONF_ABOUT_1 = "ABOUT_1";
	protected static final String CONF_ABOUT_2 = "ABOUT_2";
	protected static final String CONF_ABOUT_3 = "ABOUT_3";
	protected static final String CONF_ABOUT_4 = "ABOUT_4";
	protected static final String CONF_ABOUT_5 = "ABOUT_5";
	protected static final String CONF_ABOUT_6 = "ABOUT_6";
	protected static final String CONF_ABOUT_7 = "ABOUT_7";
	protected static final String CONF_ABOUT_8 = "ABOUT_8";
	protected static final String CONF_ABOUT_9 = "ABOUT_9";
	protected static final String CONF_ABOUT_10 = "ABOUT_10";

	protected static final String APPLICATION_NAME = "Chess PDF browser";
	protected static final String VERSION = "v1.0";
	protected static final String EMAIL_ADDRESS = "frojasg1@hotmail.com";
	protected static final String RELEASE_DATE_STRING_IN_SPANISH_FORMAT = "19/08/2016";

	/**
	 * Creates new form JDialAbout
	 */
	public AboutJDialog(java.awt.Frame parent, boolean modal,
						BaseApplicationConfigurationInterface applicationConfiguration)
	{
		super(parent, modal, applicationConfiguration, RESOURCE_FOR_ABOUT_ICON, CONFIGURATION_BASE_FILE_NAME );

//		initComponents();

		initOwnComponents();
	}

	protected void initOwnComponents()
	{
		registerInternationalString( CONF_ABOUT_1, "By Francisco Javier Rojas Garrido" );
		registerInternationalString( CONF_ABOUT_2, "" );
		registerInternationalString( CONF_ABOUT_3, "Application that helps you read chess books in PDF format allowing you to extract the chess games of the books to PGN format" );
		registerInternationalString( CONF_ABOUT_4, "" );
		registerInternationalString( CONF_ABOUT_5, "" );
		registerInternationalString( CONF_ABOUT_6, "Thanks to the authors of ChessX application, which has been a source of inspiration" );
		registerInternationalString( CONF_ABOUT_7, "Thanks to http://stackoverflow.com for the great help that they provide to developers" );
		registerInternationalString( CONF_ABOUT_8, "Thanks to Rob Camick for the post: https://tips4java.wordpress.com/2008/11/10/table-column-adjuster/ which has been used in this application." );
		registerInternationalString( CONF_ABOUT_9, "" );
		registerInternationalString( CONF_ABOUT_10, "For bug reports or comments, please send an e-mail to:" );

		setData( APPLICATION_NAME, VERSION, EMAIL_ADDRESS, RELEASE_DATE_STRING_IN_SPANISH_FORMAT );

		Dimension imageDimensions = getSizeOfImageJPanel();
		int xx = (int) (imageDimensions.getWidth() / 2);
		int yyIncrement = (int) (imageDimensions.getHeight() / 8);

		addTextLine( new ImageJPanel.TextLine( "Chess PDF", new Point( xx, yyIncrement ) ) );
		addTextLine( new ImageJPanel.TextLine( "browser", new Point( xx, yyIncrement * 3 ) ) );
		addTextLine( new ImageJPanel.TextLine( VERSION, new Point( xx, yyIncrement * 5 ) ) );

		String pathToRtfFile = System.getProperty("user.home") + File.separator  +
							getAppliConf().getConfigurationMainFolder() + File.separator +
							getAppliConf().getApplicationName() + File.separator +
							getAppliConf().getApplicationGroup() + File.separator +
							getAppliConf().getLanguage();

		setRtfDocument( getAppliConf().getInternationalPropertiesPathInJar(),
						getAppliConf().getLanguage(),
						ABOUT_RTF_FILE_NAME,
						pathToRtfFile );
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setName("AboutJDialog"); // NOI18N

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 400, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 300, Short.MAX_VALUE)
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

	/**
	 * @param args the command line arguments
	 */
	public static void main(String args[]) {
		/* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
		 * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
		 */
		try {
			for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
				if ("Nimbus".equals(info.getName())) {
					javax.swing.UIManager.setLookAndFeel(info.getClassName());
					break;
				}
			}
		} catch (ClassNotFoundException ex) {
			java.util.logging.Logger.getLogger(AboutJDialog.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (InstantiationException ex) {
			java.util.logging.Logger.getLogger(AboutJDialog.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (IllegalAccessException ex) {
			java.util.logging.Logger.getLogger(AboutJDialog.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (javax.swing.UnsupportedLookAndFeelException ex) {
			java.util.logging.Logger.getLogger(AboutJDialog.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		}
        //</editor-fold>
        //</editor-fold>
        //</editor-fold>
        //</editor-fold>

		/* Create and display the dialog */
		java.awt.EventQueue.invokeLater(new Runnable() {
			public void run() {
/*				AboutJDialog dialog = new AboutJDialog(new javax.swing.JFrame(), true);
				dialog.addWindowListener(new java.awt.event.WindowAdapter() {
					@Override
					public void windowClosing(java.awt.event.WindowEvent e) {
						System.exit(0);
					}
				});
				dialog.setVisible(true);
*/			}
		});
	}

    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
}
