/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/gpl-3.0.txt
 *
 */
package com.frojasg1.chesspdfbrowser.application;

import com.frojasg1.chesspdfbrowser.configuration.AppStringsConf;
import com.frojasg1.chesspdfbrowser.configuration.ApplicationConfiguration;
import com.frojasg1.chesspdfbrowser.io.file.implementation.PgnChessFile;
import com.frojasg1.chesspdfbrowser.model.chess.configuration.ChessLanguageConfiguration;
import com.frojasg1.chesspdfbrowser.model.chess.game.ChessGame;
import com.frojasg1.chesspdfbrowser.threads.LoadControllerInterface;
import com.frojasg1.chesspdfbrowser.threads.ScanPDFgamesThread;
import com.hotmail.frojasg1.applications.common.components.internationalization.InternException;
import com.hotmail.frojasg1.applications.common.components.internationalization.window.InternationalizedJDialog;
import com.frojasg1.general.DateFunctions;
import com.frojasg1.general.dialogs.DialogsWrapper;
import com.frojasg1.general.exceptions.ConfigurationException;
import com.frojasg1.general.exceptions.ValidateException;
import com.frojasg1.general.progress.CancellationException;
import com.frojasg1.general.progress.GeneralUpdatingProgress;
import com.frojasg1.general.progress.OperationCancellation;
import com.frojasg1.general.progress.UpdatingProgress;
import com.frojasg1.generic.GenericFunctions;
import java.awt.Component;
import java.util.Date;
import java.util.List;
import java.util.TimeZone;
import java.util.Vector;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import org.apache.pdfbox.pdmodel.PDDocument;

/**
 *
 * @author Usuario
 */
public class PDFScanProgressJDial extends InternationalizedJDialog
								implements UpdatingProgress,
												LoadControllerInterface
{
	public static final String sa_configurationBaseFileName = "PDFScanProgressWindow";

	protected Component _parent = null;
	
	protected ScanPDFgamesThread _thread = null;
	protected LoadControllerInterface _gameLoadController = null;

	protected long _beginTimeStamp = 0;

	protected int _totalNumberOfPages = 0;

//	protected UpdatingProgress _updatingProgress = null;
	protected OperationCancellation _operationCancellation = null;

	protected int _numberOfEndProgress = 0;
	
	/**
	 * Creates new form PDFScanProgressJDial
	 */
	public PDFScanProgressJDial(java.awt.Frame parent, boolean modal,
								LoadControllerInterface gameLoadController,
								PDDocument document )
	{
		super(parent, modal, ApplicationConfiguration.instance());
		initComponents();

		_parent = parent;

		setWindowConfiguration( );

		boolean initialConfiguration = true;
		fillInComboBoxesOfChessLanguage( initialConfiguration );

		try
		{
			changeLanguage( getAppliConf().getLanguage() );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}

		_gameLoadController = gameLoadController;
		_totalNumberOfPages = document.getNumberOfPages();

		boolean hasToCancel = false;
		_operationCancellation = new OperationCancellation(hasToCancel);
/*
		GeneralUpdatingProgress gup = new GeneralUpdatingProgress( );
		gup.up_setParentUpdatingProgress( this );
		gup.up_setOperationCancellation( _operationCancellation );
		gup.up_prepareNextSlice( _totalNumberOfPages );
		_updatingProgress = gup;
*/
//		_thread = new ScanPDFgamesThread( this, document, _updatingProgress );
		_thread = new ScanPDFgamesThread( this, document, this );

		jPB_progress.setMinimum( 0 );
		jPB_progress.setMaximum( (int) _totalNumberOfPages );

		_thread.start();
	}

	protected void setWindowConfiguration( )
	{
/*
		a_intern = new JFrameInternationalization(	ApplicationConfiguration.sa_MAIN_FOLDER,
													ApplicationConfiguration.sa_APPLICATION_NAME,
													ApplicationConfiguration.sa_CONFIGURATION_GROUP,
													ApplicationConfiguration.sa_PROPERTIES_PATH_IN_JAR,
													a_configurationBaseFileName,
													this,
													parent,
													a_vectorJpopupMenus,
													false,
													mapRRCI );
*/
		createInternationalization(	getAppliConf().getConfigurationMainFolder(),
									getAppliConf().getApplicationNameFolder(),
									getAppliConf().getApplicationGroup(),
									getAppliConf().getInternationalPropertiesPathInJar(),
									sa_configurationBaseFileName,
									this,
									_parent,
									null,
									true,
									null );

//		setIcon( "com/frojasg1/chesspdfbrowser/resources/icons/App.icon.png" );
	}
	
	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        jB_cancel = new javax.swing.JButton();
        jB_start = new javax.swing.JButton();
        jP_language = new javax.swing.JPanel();
        jL_language = new javax.swing.JLabel();
        jCB_languageToParseGamesFrom = new javax.swing.JComboBox();
        jTF_languageToParseGamesFromPiecesString = new javax.swing.JTextField();
        jP_progress = new javax.swing.JPanel();
        jPB_progress = new javax.swing.JProgressBar();
        jL_current = new javax.swing.JLabel();
        jL_currentProgress = new javax.swing.JLabel();
        jL_tiempo = new javax.swing.JLabel();
        jL_timeProgress = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Scan for games");
        setName("PDFscanProgressDialog"); // NOI18N
        setPreferredSize(new java.awt.Dimension(400, 215));
        setResizable(false);
        getContentPane().setLayout(null);

        jPanel1.setName("kk"); // NOI18N
        jPanel1.setLayout(null);

        jB_cancel.setText("Cancel");
        jB_cancel.setName("jB_cancel"); // NOI18N
        jB_cancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jB_cancelActionPerformed(evt);
            }
        });
        jPanel1.add(jB_cancel);
        jB_cancel.setBounds(210, 150, 110, 23);

        jB_start.setText("Start");
        jB_start.setName("jB_start"); // NOI18N
        jB_start.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jB_startActionPerformed(evt);
            }
        });
        jPanel1.add(jB_start);
        jB_start.setBounds(80, 150, 110, 23);

        jP_language.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)), "Language of games"));
        jP_language.setName("jP_language"); // NOI18N
        jP_language.setLayout(null);

        jL_language.setText("Language :");
        jL_language.setName("jL_language"); // NOI18N
        jP_language.add(jL_language);
        jL_language.setBounds(10, 20, 110, 14);

        jCB_languageToParseGamesFrom.setName(""); // NOI18N
        jCB_languageToParseGamesFrom.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCB_languageToParseGamesFromActionPerformed(evt);
            }
        });
        jP_language.add(jCB_languageToParseGamesFrom);
        jCB_languageToParseGamesFrom.setBounds(130, 20, 140, 20);

        jTF_languageToParseGamesFromPiecesString.setName(""); // NOI18N
        jTF_languageToParseGamesFromPiecesString.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                jTF_languageToParseGamesFromPiecesStringKeyReleased(evt);
            }
        });
        jP_language.add(jTF_languageToParseGamesFromPiecesString);
        jTF_languageToParseGamesFromPiecesString.setBounds(280, 20, 80, 20);

        jPanel1.add(jP_language);
        jP_language.setBounds(0, 90, 390, 50);

        jP_progress.setBorder(javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)), "Progress"));
        jP_progress.setName("jP_progress"); // NOI18N
        jP_progress.setLayout(null);

        jPB_progress.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
        jPB_progress.setStringPainted(true);
        jP_progress.add(jPB_progress);
        jPB_progress.setBounds(50, 20, 290, 30);

        jL_current.setText("current :");
        jL_current.setName("jL_current"); // NOI18N
        jP_progress.add(jL_current);
        jL_current.setBounds(50, 60, 60, 20);

        jL_currentProgress.setText("0 / 0");
        jP_progress.add(jL_currentProgress);
        jL_currentProgress.setBounds(120, 60, 90, 20);

        jL_tiempo.setText("Time :");
        jL_tiempo.setName("jL_tiempo"); // NOI18N
        jP_progress.add(jL_tiempo);
        jL_tiempo.setBounds(220, 60, 50, 20);

        jL_timeProgress.setText("0:00:00");
        jL_timeProgress.setName(""); // NOI18N
        jP_progress.add(jL_timeProgress);
        jL_timeProgress.setBounds(280, 60, 60, 20);

        jPanel1.add(jP_progress);
        jP_progress.setBounds(0, 0, 390, 90);

        getContentPane().add(jPanel1);
        jPanel1.setBounds(0, 0, 390, 180);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void jB_cancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jB_cancelActionPerformed
        // TODO add your handling code here:

        doCancel();
    }//GEN-LAST:event_jB_cancelActionPerformed

    private void jB_startActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jB_startActionPerformed
        // TODO add your handling code here:

        doStart();
    }//GEN-LAST:event_jB_startActionPerformed

    private void jCB_languageToParseGamesFromActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCB_languageToParseGamesFromActionPerformed
        // TODO add your handling code here:

        updateNewComboSelection( (JComboBox) evt.getSource() );
    }//GEN-LAST:event_jCB_languageToParseGamesFromActionPerformed

    private void jTF_languageToParseGamesFromPiecesStringKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jTF_languageToParseGamesFromPiecesStringKeyReleased
        // TODO add your handling code here:

        JTextField jtf = (JTextField) evt.getComponent();

        System.out.println( "Name : " + jtf.getName() );
        System.out.println( "Text : " + jtf.getText() );
    }//GEN-LAST:event_jTF_languageToParseGamesFromPiecesStringKeyReleased

	/**
	 * @param args the command line arguments
	 */
	public static void main(String args[]) {
		/* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
		 * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
		 */
		try {
			for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
				if ("Nimbus".equals(info.getName())) {
					javax.swing.UIManager.setLookAndFeel(info.getClassName());
					break;
				}
			}
		} catch (ClassNotFoundException ex) {
			java.util.logging.Logger.getLogger(PDFScanProgressJDial.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (InstantiationException ex) {
			java.util.logging.Logger.getLogger(PDFScanProgressJDial.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (IllegalAccessException ex) {
			java.util.logging.Logger.getLogger(PDFScanProgressJDial.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (javax.swing.UnsupportedLookAndFeelException ex) {
			java.util.logging.Logger.getLogger(PDFScanProgressJDial.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		}
        //</editor-fold>

		/* Create and display the dialog */
		java.awt.EventQueue.invokeLater(new Runnable() {
			public void run() {
/*				PDFScanProgressJDial dialog = new PDFScanProgressJDial(new javax.swing.JFrame(), true);
				dialog.addWindowListener(new java.awt.event.WindowAdapter() {
					@Override
					public void windowClosing(java.awt.event.WindowEvent e) {
						System.exit(0);
					}
				});
				dialog.setVisible(true);
*/			}
		});
	}

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jB_cancel;
    private javax.swing.JButton jB_start;
    private javax.swing.JComboBox jCB_languageToParseGamesFrom;
    private javax.swing.JLabel jL_current;
    private javax.swing.JLabel jL_currentProgress;
    private javax.swing.JLabel jL_language;
    private javax.swing.JLabel jL_tiempo;
    private javax.swing.JLabel jL_timeProgress;
    private javax.swing.JProgressBar jPB_progress;
    private javax.swing.JPanel jP_language;
    private javax.swing.JPanel jP_progress;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JTextField jTF_languageToParseGamesFromPiecesString;
    // End of variables declaration//GEN-END:variables

	protected void fillInOneComboBoxOfChessLanguage( JComboBox combo,
													Vector< ChessLanguageConfiguration.LanguageConfigurationData > vector,
													String defaultValue )
	{
		ChessLanguageConfiguration.LanguageConfigurationData selectedItem = null;
		if( defaultValue != null )
		{
			selectedItem = ChessLanguageConfiguration.getLanguageConfigurationData( defaultValue );
		}
		else if( ( combo.getModel() != null ) && ( combo.getModel().getSize() > 0 ) )
		{
			selectedItem = ( ChessLanguageConfiguration.LanguageConfigurationData ) combo.getSelectedItem();
		}
		DefaultComboBoxModel<ChessLanguageConfiguration.LanguageConfigurationData> dcbm =
			new DefaultComboBoxModel<ChessLanguageConfiguration.LanguageConfigurationData>( vector );
		combo.setModel( dcbm );
		combo.setSelectedItem( selectedItem );
	}

	protected void fillInComboBoxesOfChessLanguage( boolean initialConfiguration )
	{
		Vector< ChessLanguageConfiguration.LanguageConfigurationData > vector = ChessLanguageConfiguration.getListOfLanguages();

		fillInOneComboBoxOfChessLanguage( jCB_languageToParseGamesFrom, vector,
											( initialConfiguration ?
												getAppliConf().getConfigurationOfChessLanguageToParseTextFrom() :
												null ) );
	}

	protected String getChessLanguageForConfiguration( JComboBox combo )
	{
		String result = null;

		if( combo.getSelectedItem() != null )
		{
			ChessLanguageConfiguration.LanguageConfigurationData lcd =
				(ChessLanguageConfiguration.LanguageConfigurationData) combo.getSelectedItem();

			if( lcd == ChessLanguageConfiguration.getCustomLanguage() )
			{
				result = jTF_languageToParseGamesFromPiecesString.getText();
			}
			else
				result = lcd._languageName;
		}

		return( result );
	}
	
	protected void updateNewComboSelection( JComboBox combo )
	{
		ChessLanguageConfiguration.LanguageConfigurationData lcd =
			(ChessLanguageConfiguration.LanguageConfigurationData) combo.getSelectedItem();

		if( lcd == ChessLanguageConfiguration.getCustomLanguage() )
		{
			jTF_languageToParseGamesFromPiecesString.setText( "" );
			jTF_languageToParseGamesFromPiecesString.setEnabled( true );
			jTF_languageToParseGamesFromPiecesString.requestFocus( true );
		}
		else
		{
			jTF_languageToParseGamesFromPiecesString.setText( lcd._stringOfPieceCodes );
			jTF_languageToParseGamesFromPiecesString.setEnabled( false );
		}
	}

	protected void doCancel()
	{
		if( ( _numberOfEndProgress <= 2 ) && _thread.isAlive() )
		{
			_operationCancellation.M_setHasToCancel( true );
		}
		else
		{
			boolean closeWindow = true;
			formWindowClosing( closeWindow );
		}
	}

	protected void applyChanges()
	{
		ApplicationConfiguration appConf = ApplicationConfiguration.instance();
		
		String chessLanguageToParseFrom = getChessLanguageForConfiguration( jCB_languageToParseGamesFrom );
		if( chessLanguageToParseFrom != null )
			appConf.setConfigurationOfChessLanguageToParseTextFrom( chessLanguageToParseFrom );
	}

	protected void doStart()
	{
		try
		{
			boolean proceed = false;
			try
			{
				validateChessLanguageComboBoxChanges();
				proceed = true;
			}
			catch( ValidateException ve )
			{
				GenericFunctions.instance().getDialogsWrapper().showMessageDialog( this, ve.getMessage(),
												getAppStrConf().getProperty( AppStringsConf.CONF_CONFIGURATION_ERROR ),
												DialogsWrapper.ERROR_MESSAGE );

				Component comp = ve.getComponentNotValidated();
				if( comp != null )
				{
					boolean showFocus = true;
					a_intern.setFocus( comp, showFocus );
				}
			}

			if( proceed )
			{
				applyChanges();

				jCB_languageToParseGamesFrom.setEnabled( false );
				jTF_languageToParseGamesFromPiecesString.setEnabled( false );
				jB_start.setEnabled( false );
				
				_thread.letsStart();
			}
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}
	}

	@Override
	public void newChessListGameLoaded(List<ChessGame> list, PgnChessFile pgnFile)
	{
		_gameLoadController.newChessListGameLoaded( list, pgnFile );
	}

	@Override
	public void newChessPDFLoaded(PDDocument pdfDocument)
	{
	}

	@Override
	public void cancel()
	{
		boolean closeWindow = true;
		formWindowClosing( closeWindow );
	}

	@Override
	public void block()
	{
		_gameLoadController.block();
	}

	@Override
	public void unblock()
	{
		_gameLoadController.unblock();
	}

	@Override
	public void showError(String message, String title)
	{
		_gameLoadController.showError( message, title );
	}

	protected void updateLabels( long completion )
	{
		jL_currentProgress.setText( completion + " / " + _totalNumberOfPages );

		Date now = new Date();
		long elapsedTime = now.getTime() - _beginTimeStamp;

		Date elapsedTimeDate = new Date( elapsedTime );

		String timeString = DateFunctions.formatDate( elapsedTimeDate, "HH:mm:ss",
														TimeZone.getTimeZone("GMT") );

		jL_timeProgress.setText( timeString );
	}

	@Override
	public void formWindowClosing( boolean closeWindow )
	{
		super.formWindowClosing( closeWindow );

		while( _thread.isAlive() )
		{
			try
			{
				Thread.sleep( 100 );
			}
			catch( Throwable th )
			{
				th.printStackTrace();
			}
		}
		_thread = null;

		setVisible( false );
	}

	public void validateChessLanguageComboBoxChanges( ) throws ValidateException
	{
		if( jCB_languageToParseGamesFrom.getSelectedItem() != null )
		{
			ChessLanguageConfiguration.LanguageConfigurationData lcd =
				(ChessLanguageConfiguration.LanguageConfigurationData) jCB_languageToParseGamesFrom.getSelectedItem();

			if( lcd == ChessLanguageConfiguration.getCustomLanguage() )
			{
				ChessLanguageConfiguration clc = ChessLanguageConfiguration.getConfiguration( jTF_languageToParseGamesFromPiecesString.getText() );
						
				if( clc == null )
				{
					try
					{
						// if the data to create a new language configuration is not valid, it will throw an exception.
						ChessLanguageConfiguration.createConfiguration(null, jTF_languageToParseGamesFromPiecesString.getText());
					}
					catch( Throwable th )
					{
						th.printStackTrace();
						throw( new ValidateException( th.getMessage(), jTF_languageToParseGamesFromPiecesString ) );
					}
				}
			}
		}
	}

/*
	@Override
	public boolean hasBeenCancelled()
	{
		return( _operationCancellation.getHasToCancel() );
	}
*/
	@Override
	public void changeLanguage( String language ) throws ConfigurationException, InternException
	{
		super.changeLanguage(language);

		jB_cancel.setText( getAppStrConf().getProperty( AppStringsConf.CONF_CANCEL ) );
	}

	protected AppStringsConf getAppStrConf()
	{
		return( AppStringsConf.instance() );
	}

	public ApplicationConfiguration getAppliConf()
	{
		return( (ApplicationConfiguration) super.getAppliConf() );
	}

	@Override
	public void up_childStarts()
	{
		if( _beginTimeStamp == 0 )
		{
			Date ts = new Date();
			_beginTimeStamp = ts.getTime();
		}

		updateProgress( 0 );
	}

	@Override
	public void up_updateProgressFromChild(double completedOverOne) throws CancellationException
	{
		updateProgress( (int) (completedOverOne * _totalNumberOfPages) );
	}

	protected void updateProgress( int completion )
	{
		jPB_progress.setValue( (int) completion );

		updateLabels( completion );
	}
	
	@Override
	public void up_childEnds() throws CancellationException
	{
		try
		{
			updateProgress( _totalNumberOfPages );
		}
		catch( Throwable th )
		{}

		if( _numberOfEndProgress > 1 )
			jB_cancel.setText( getAppStrConf().getProperty( AppStringsConf.CONF_CLOSE ) );

		_numberOfEndProgress++;
	}

	@Override
	public OperationCancellation up_getOperationCancellation()
	{
		return( _operationCancellation );
	}

	@Override
	public void up_setDebug(boolean debug)
	{
//		throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
	}

	@Override
	public void formWindowClosingEvent( )
	{
		doCancel();
	}
}
