/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/gpl-3.0.txt
 *
 */
package com.frojasg1.chesspdfbrowser.threads;

import com.frojasg1.chesspdfbrowser.configuration.ApplicationConfiguration;
import com.frojasg1.chesspdfbrowser.io.parsers.implementation.RawChessGameParser;
import com.frojasg1.chesspdfbrowser.model.chess.game.ChessGame;
import com.frojasg1.chesspdfbrowser.pdf.renderer.MyPDFRenderer;
import com.frojasg1.chesspdfbrowser.pdf.textsegmentation.implementation.PDFPageSegmentator;
import com.frojasg1.general.progress.CancellationException;
import com.frojasg1.general.progress.GeneralUpdatingProgress;
import com.frojasg1.general.progress.UpdatingProgress;
import java.util.List;
import javax.swing.SwingUtilities;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.rendering.PDFRenderer;

/**
 *
 * @author Usuario
 */
public class ScanPDFgamesThread extends Thread
{
	protected LoadControllerInterface _controller = null;
	protected List<ChessGame> _list = null;

	protected PDDocument _pdfDocument = null;
	protected PDFRenderer _pdfRenderer = null;

	protected GeneralUpdatingProgress _gup = null;

	protected boolean _letsStart = false;

	public ScanPDFgamesThread( LoadControllerInterface controller, PDDocument document, UpdatingProgress up )
	{
		_controller = controller;
		_pdfDocument = document;

		_gup = new GeneralUpdatingProgress( );
		_gup.up_setParentUpdatingProgress( up );
//		gup.up_prepareNextSlice( _totalNumberOfPages );
	}

	public void letsStart()
	{
		_letsStart = true;
	}

	@Override
	public void run()
	{
		boolean cancelDone = false;

		while( !_letsStart && !cancelDone )
		{
			try
			{
				Thread.sleep( 100 );
			}
			catch( Throwable th )
			{
				th.printStackTrace();
			}

			cancelDone = false;
			if( ( _gup != null ) && ( _gup.up_getOperationCancellation() != null ) )
				cancelDone = _gup.up_getOperationCancellation().M_getHasToCancel();
		}

		if( ! cancelDone )
		{
			try
			{
				ApplicationConfiguration conf = ApplicationConfiguration.instance();
				if( _controller != null )	_controller.block();

				PDFPageSegmentator pageSegmentator = null;
				try
				{
					_pdfRenderer = new MyPDFRenderer( _pdfDocument );

					pageSegmentator = new PDFPageSegmentator( conf );
					pageSegmentator.setChessViewConfiguration( conf );

					_gup.up_reset( 1.0D );
					_gup.up_prepareNextSlice( 1.0D );
					pageSegmentator.initialize(_pdfDocument, 3, 30, _gup );
					_gup.up_performEnd();
				}
				catch( CancellationException ce )
				{
					cancelDone = true;
				}
				catch( Throwable th )
				{
					th.printStackTrace();
					if( _controller != null )
					{
						_controller.showError( "Error scanning pdf. ERROR: " + th.getMessage(), "Error scanning pdf" );
					}
				}

				try
				{
//					_gup.up_prepareNextSlice( 1.0D );
//					_gup.up_reset( 1.0D );
//					_gup.up_prepareNextSlice( 1.0D );
					RawChessGameParser rcgp = new RawChessGameParser( ApplicationConfiguration.instance().getChessLanguageConfigurationToParseTextFrom(),
																		_gup );
//					_gup.up_performEnd();

					_list = rcgp.parseChessGameText( null, pageSegmentator );

					if( _controller != null )
						_controller.newChessListGameLoaded(_list, null); 
				}
				catch( CancellationException ce )
				{
					cancelDone = true;
				}
				catch( Throwable th )
				{
					th.printStackTrace();
					if( _controller != null )
					{
						_controller.showError( "ERROR: " + th.getMessage(), "Error scanning pdf" );
					}
				}
			}
			finally
			{
				if( _controller != null )
					_controller.unblock();
			}
		}

		if( cancelDone && ( _controller != null ) )
		{
			SwingUtilities.invokeLater( new Thread(){
				public void run()
				{
					_controller.cancel();
				}
			});
		}
	}

	public List<ChessGame> getGameList()
	{
		return( _list );
	}
}
