/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.chesspdfbrowser.io.parsers.tokens;

import com.frojasg1.chesspdfbrowser.configuration.ChessStringsConf;
import com.frojasg1.chesspdfbrowser.exception.ChessParserException;
import com.frojasg1.chesspdfbrowser.model.chess.game.tree.NAG;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 *
 * @author Usuario
 */
public class MoveToken extends Token
{
	protected int _moveTurnNumber;
	protected boolean _isWhiteToMove;
	protected String _comment = null;
	protected List<NAG> _nagList = null;

	// the moveString must be in English.
	
	public MoveToken()
	{
		super( TokenId.MOVE );
		
		_nagList = new ArrayList<NAG>();
	}

	public MoveToken( PreToken preToken ) throws ChessParserException
	{
		super( preToken );

		if( getTokenId() != TokenId.MOVE )
			throw( new ChessParserException( getChessStrConf().getProperty( ChessStringsConf.CONF_NOT_A_MOVE_PRETOKEN ) ) );

		_nagList = new ArrayList<NAG>();
	}

	public void setMoveNumber( int moveNumber )		{	_moveTurnNumber = moveNumber;	}
	public void setIsWhiteToMove( boolean value )	{	_isWhiteToMove = value;	}
	public void setComment( String comment )		{	_comment = comment;	}

	public int getMoveNumber()						{	return( _moveTurnNumber );	}
	public boolean isWhiteToMove()					{	return( _isWhiteToMove );	}
	public String getComment()						{	return( _comment );	}

	public int getNumberOfPly()
	{
		int result = 2 * ( _moveTurnNumber - 1 ) + 1;
		if( !_isWhiteToMove )	result++;

		return( result );
	}

	public void clearNAGlist()
	{
		_nagList.clear();
	}
	
	public void addNag( NAG nag )
	{
		_nagList.add(nag);
	}
	
	public void addAllNAGs( Iterator<NAG> it )
	{
		while( it.hasNext() )
			_nagList.add(it.next());
	}
	
	public Iterator<NAG> nagIterator()
	{
		return( _nagList.iterator() );
	}
	
	public String toString()
	{
		String result = String.format( "%d. %s, isWhiteToMove: %s",
										getMoveNumber(), getString(),
										String.valueOf( isWhiteToMove() ) );

		return( result );
	}

	protected static ChessStringsConf getChessStrConf()
	{
		return( ChessStringsConf.instance() );
	}
}
