/* 
 * Copyright (C) 2017 Francisco Javier Rojas Garrido <frojasg1@hotmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3.0 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 */
package com.frojasg1.general.desktop.view.about;

import com.frojasg1.applications.common.configuration.application.BaseApplicationConfigurationInterface;
import com.frojasg1.general.DateFunctions;
import com.frojasg1.general.ResourceFunctions;
import com.frojasg1.general.desktop.files.FileFunctions;
import com.frojasg1.general.number.IntegerFunctions;
import com.frojasg1.general.desktop.view.pdf.ImageJPanel;
import com.frojasg1.desktop.liblens.graphics.lens.Lens;
import com.frojasg1.desktop.liblens.graphics.lens.LensJPanel;
import com.hotmail.frojasg1.applications.common.components.internationalization.window.InternationalizedJDialog;
import java.awt.Color;
import java.awt.Desktop;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.image.BufferedImage;
import java.io.File;
import static java.lang.Thread.sleep;
import java.net.URI;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import javax.swing.JScrollPane;
import javax.swing.text.Document;

/**
 *
 * @author Usuario
 */
public class GenericAboutJDialog extends InternationalizedJDialog
								implements AboutJPanelControllerInterface
{
	protected static final String CONF_SEND_EMAIL_TO = "SEND_EMAIL_TO";
	protected static final String CONF_RELEASED_ON = "RELEASED_ON";
	protected static final String CONF_TITLE = "TITLE";
	protected static final String BASE_OF_CONF_ABOUT_ = "ABOUT_";

//	protected static final String DEFAULT_LANGUAGE = "EN";

	protected GenericAboutJPanel _aboutJPanel = null;
	protected LensJPanel _lensJPanel = null;

	protected Object _mutex = new Object();

	protected String _applicationName = null;
	protected String _version = null;
	protected String _configurationBaseFileName = "JDial_about";

	protected String _emailAddress = null;

	protected SelectionThread _selectionThread = null;
	protected String _releaseDateInSpanishFormat = null;

	protected int _oldScrollYYValue = -1;

	protected class SelectionThread extends Thread
	{
		protected boolean _hasToStart = false;
		protected boolean _hasToStop = false;
		protected static final long sa_sleepTime = 33;
		protected static final int sa_increment = 1;

		public SelectionThread(  )
		{
			super();
		}

		public void M_start()
		{
			_hasToStart = true;
		}
		
		public void M_stop()
		{
			_hasToStop = true;
		}

		public void run()
		{
			try
			{
				while( ! _hasToStart && ! _hasToStop )
				{
					try
					{
						sleep( sa_sleepTime );
					}
					catch( Throwable th )
					{
						th.printStackTrace();
					}
				}

				while( ! _hasToStop )
				{
					for( int ii=0; (ii<getText().length()-2-sa_increment) && !_hasToStop; ii = ii + sa_increment )
					{
						synchronized( _mutex )
						{
							int position = IntegerFunctions.min( ii, getText().length()-2-sa_increment );

							_oldScrollYYValue = -1;

							JScrollPane jsp = null;
							if( _aboutJPanel.getJTextComponent().getParent().getParent() instanceof JScrollPane )
							{
								jsp = (JScrollPane) _aboutJPanel.getJTextComponent().getParent().getParent();
								_oldScrollYYValue = jsp.getVerticalScrollBar().getValue();
							}
							
							_aboutJPanel.getJTextComponent().setCaretPosition( position + 2 );
							setSelectionStart( position );
							setSelectionEnd( position + 2 );

//							System.out.println( "position: " + position );

							java.awt.EventQueue.invokeLater(new Runnable() {
								public void run() {
									try
									{
										_aboutJPanel.changeUrlCursor(_oldScrollYYValue);
									}
									catch( Throwable th )
									{

									}
								} } );

							repaint();
						}

						try
						{
							sleep( sa_sleepTime );
						}
						catch( Throwable th )
						{
							th.printStackTrace();
						}
					}
				}
			}
			catch( Throwable th )
			{
				th.printStackTrace();
			}
		}
	}


	/**
	 * Creates new form GenericAboutJDialog
	 */
	public GenericAboutJDialog(java.awt.Frame parent, boolean modal,
								BaseApplicationConfigurationInterface applicationConfiguration,
								String pathInJarForApplicationImage, String configurationBaseFileName )
	{
		super(parent, modal, applicationConfiguration);

		if( configurationBaseFileName != null )
			_configurationBaseFileName =  configurationBaseFileName;

		initComponents();

		initOwnComponents( pathInJarForApplicationImage );

		setWindowConfiguration( );

		if( modal )
			setAlwaysOnTop( true );
	}

	public void setData( String applicationName, String version, String emailAddress, String releaseDateInSpanishFormat )
	{
		_applicationName = applicationName;
		_version = version;
		_emailAddress = emailAddress;
		_releaseDateInSpanishFormat = releaseDateInSpanishFormat;

		initComponentContents();
	}

	protected BufferedImage getImage( String pathInJarForApplicationImage )
	{
		BufferedImage result = null;
		try
		{
			result = ResourceFunctions.loadResourceImage(pathInJarForApplicationImage);
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}

		return( result );
	}

	protected void initOwnComponents( String pathInJarForApplicationImage )
	{
		_aboutJPanel = new GenericAboutJPanel( this, getImage( pathInJarForApplicationImage ) );
		_aboutJPanel.setPreferredSize( jPanel1.getSize() );

		_lensJPanel = new LensJPanel( _aboutJPanel, 125, Lens.SA_MODE_AMPLIFY, true, _mutex );

		_aboutJPanel.setLensJPanel( _lensJPanel );

		jPanel1.add( _lensJPanel );
		_lensJPanel.setBounds( 0, 0, jPanel1.getWidth(), jPanel1.getHeight() );

		_aboutJPanel.getJTextComponent().requestFocusInWindow();
	}

	protected void initComponentContents()
	{
		setTitle( getInternationalString(CONF_TITLE) );

		String text = createAboutText();
		_aboutJPanel.setText( text );

		_aboutJPanel.setSendEmailButtonText(getInternationalString(CONF_SEND_EMAIL_TO) + " " +
												_emailAddress );		
	}

	protected void setWindowConfiguration( )
	{
		createInternationalization(getAppliConf().getConfigurationMainFolder(),
									getAppliConf().getApplicationNameFolder(),
									getAppliConf().getApplicationGroup(),
									getAppliConf().getInternationalPropertiesPathInJar(),
									_configurationBaseFileName,
									this,
									getParent(),
									null,
									true,
									null );

		registerInternationalString(CONF_SEND_EMAIL_TO, "Send e-mail to:" );
		registerInternationalString(CONF_RELEASED_ON, "Released on" );
		registerInternationalString(CONF_TITLE, "About" );

//		setIcon( "com/frojasg1/chesspdfbrowser/resources/icons/App.icon.png" );
	}

	protected String getText()
	{
		String result = "";

		if( ( _aboutJPanel != null ) && ( _aboutJPanel.getJTextComponent() != null ) )
		{
			int length = _aboutJPanel.getJTextComponent().getDocument().getLength();
			try
			{
				result = _aboutJPanel.getJTextComponent().getDocument().getText( 0, length );
			}
			catch( Throwable th )
			{
				th.printStackTrace();
			}
		}

		return( result );
	}

	public void setSelectionStart( int position )
	{
		_aboutJPanel.getJTextComponent().setSelectionStart( position );
	}

	public void setSelectionEnd( int position )
	{
		_aboutJPanel.getJTextComponent().setSelectionEnd( position );
	}

	public void setRtfDocument( Document doc )
	{
		synchronized( _mutex )  // because of the setSelectionStart and End in the setRtfDocument of _aboutJPanel, and may be the changes in the view.
		{
			_aboutJPanel.setRtfDocument(doc);
		}
	}

	public void setRtfDocument( String resourcePath, String language, String fileName, String pathToFile )
	{
		Document doc = null;

		String longFileName = pathToFile + File.separator + fileName;
		doc = FileFunctions.loadRtfDocumentFromFile( longFileName );

		if( doc == null )
		{
			String longResourceName = resourcePath + "/" + language + "/" + fileName;
			doc = ResourceFunctions.loadResourceRTFDocument( longResourceName );

			if( doc == null )
			{
				longResourceName = resourcePath + "/" + getAppliConf().getDefaultLanguage() + "/" + fileName;
				doc = ResourceFunctions.loadResourceRTFDocument( longResourceName );
			}

			if( doc != null )
			{
				// if the file was not prevously created, we copy now the resource to the configuracion path.
				ResourceFunctions.copyBinaryResourceToFile( longResourceName, longFileName );
			}
		}

		setRtfDocument( doc );
	}


	/**
	 * This method is called from within the constructor to initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is always
	 * regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("About");
        setMinimumSize(new java.awt.Dimension(650, 440));
        setName("AboutJDialog"); // NOI18N
        setPreferredSize(new java.awt.Dimension(650, 440));
        setResizable(false);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });
        getContentPane().setLayout(null);

        jPanel1.setMinimumSize(new java.awt.Dimension(650, 440));
        jPanel1.setPreferredSize(new java.awt.Dimension(650, 405));

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 650, Short.MAX_VALUE)
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 440, Short.MAX_VALUE)
        );

        getContentPane().add(jPanel1);
        jPanel1.setBounds(0, 0, 650, 405);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        // TODO add your handling code here:

		exit( null );

    }//GEN-LAST:event_formWindowClosing

	/**
	 * @param args the command line arguments
	 */
	public static void main(String args[]) {
		/* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
		 * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
		 */
		try {
			for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
				if ("Nimbus".equals(info.getName())) {
					javax.swing.UIManager.setLookAndFeel(info.getClassName());
					break;
				}
			}
		} catch (ClassNotFoundException ex) {
			java.util.logging.Logger.getLogger(GenericAboutJDialog.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (InstantiationException ex) {
			java.util.logging.Logger.getLogger(GenericAboutJDialog.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (IllegalAccessException ex) {
			java.util.logging.Logger.getLogger(GenericAboutJDialog.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		} catch (javax.swing.UnsupportedLookAndFeelException ex) {
			java.util.logging.Logger.getLogger(GenericAboutJDialog.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
		}
        //</editor-fold>

		/* Create and display the dialog */
		java.awt.EventQueue.invokeLater(new Runnable() {
			public void run() {
/*				GenericAboutJDialog dialog = new GenericAboutJDialog(new javax.swing.JFrame(), true);
				dialog.addWindowListener(new java.awt.event.WindowAdapter() {
					@Override
					public void windowClosing(java.awt.event.WindowEvent e) {
						System.exit(0);
					}
				});
				dialog.setVisible(true);
*/			}
		});
	}

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel jPanel1;
    // End of variables declaration//GEN-END:variables

	public void setFrameWidth( int value )
	{
		_aboutJPanel.setFrameThick(value);
	}
	
	public void setFontForTextLines( Font font )
	{
		_aboutJPanel.setFontForTextLines(font);
	}

	public void setForegroundColorForTextLines( Color color )
	{
		_aboutJPanel.setForegroundColorForTextLines(color);
	}
	
	public void addTextLine( ImageJPanel.TextLine textLine )
	{
		_aboutJPanel.addTextLine(textLine);
	}

	public void clearTextLines()
	{
		_aboutJPanel.clearTextLines();
	}

	public Dimension getSizeOfImageJPanel()
	{
		return( _aboutJPanel.getSizeOfImageJPanel() );
	}
	
	@Override
	public void exit( Object origin )
	{
		if( _selectionThread != null )	_selectionThread.M_stop();

		while( _selectionThread.isAlive() )
		{
			try
			{
				Thread.sleep( 100 );
			}
			catch( Throwable th )
			{
				th.printStackTrace();
			}
		}
		_selectionThread = null;

		boolean closeWindow = true;
		formWindowClosing( closeWindow );
	}

	@Override
	public void sendEmailAddress()
	{
        try
        {
            Desktop.getDesktop().mail( new URI( "mailto:" + _emailAddress ) );
        }
        catch( Throwable th )
        {
            th.printStackTrace();
        }
	}

	@Override
	public void setVisible( boolean visible )
	{
		if( visible )
		{
			_selectionThread = new SelectionThread();
			_selectionThread.start();

			java.awt.EventQueue.invokeLater(new Runnable() {
				public void run() {
					try
					{
						_selectionThread.M_start();
					}
					catch( Throwable th )
					{
					}
				} } );

		}

		super.setVisible( visible );
	}

	protected Date getReleaseDate()
	{
		Date result = null;

		SimpleDateFormat sdf = new SimpleDateFormat( "dd/MM/yyyy" );
		try
		{
			result = sdf.parse( _releaseDateInSpanishFormat );
		}
		catch( Throwable th )
		{
			th.printStackTrace();
		}
/*
		Calendar calendar = new GregorianCalendar();
		calendar.set( 2016, Calendar.JANUARY, 30 );
		Date result = calendar.getTime();
*/
		return( result );
	}

	protected String createAboutText()
	{
		StringBuilder sb = new StringBuilder();
		
		String tmpStr = _applicationName + " " + _version + "\n";
		sb.append( tmpStr );

		tmpStr = getInternationalString(CONF_RELEASED_ON) + " " +
							DateFunctions.formatDate_yyyy( getReleaseDate(), DateFormat.SHORT ) + " \n";

		String baseLabel = BASE_OF_CONF_ABOUT_;
		int index=1;
		while( tmpStr != null )
		{
			sb.append( tmpStr + "\n" );
			String label = String.format( "%s%d", baseLabel, index );
			tmpStr = getInternationalString( label );
			index++;
		}

		tmpStr = " " + _emailAddress + "\n";
		sb.append( tmpStr );

		tmpStr = "                                                  ";
		sb.append( tmpStr + "\n" );
		sb.append( tmpStr );

		return( sb.toString() );
	}

}
